#================================================================#
# ETmod 2008 by McFly
#================================================================#

import es, gamethread, popuplib
from includes.includes import *
from languages.languages import *
from logs.logs import *
from classes.classes import *
from skills.skills import *
from stats.stats import *
from players.players import *
from admins.admins import *
from menus.menus import *
from server.server import *

# Information
info = es.AddonInfo()
info.name = "ETmod 2008"
info.version = "2008 Beta 0.7"
info.author = "McFly"
info.url = "http://www.mattie.info/cs/forums/index.php"
info.description = "Not so far from the famous game Enemy Territory, but for Counter-Strike:Source"

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():	
	# Register chat commands
	regSayHooks()
	
	# Register client and server commands
	regClientCmds()
	regServerCmds()
	
	# Needed config
	es.set("et_old_noisy", es.getInt("eventscripts_noisy"), "A variable used to reset 'eventscripts_noisy' when the mod will be unloaded")
	es.set("eventscripts_noisy", 1)
	es.set("et_eventscripts_previousmap", "", "Last map played")	
	
	# Prepare mod variables
	prepareVariables()
	
	# Load the sub-scripts	
	loadSubScripts()
	
	# Use server config if mod hasn't been loaded at server start
	if	str(server_var["et_eventscripts_previousmap"]) == "" or\
		str(server_var["et_eventscripts_previousmap"]) != str(server_var["eventscripts_currentmap"]):
		getModReady(server_var["eventscripts_currentmap"])
	
	# The loop for synchronized respawn
	es.set("et_mod_loaded", 1, "To remember that the mod is running")
	expansiveLoop(0)
	#cheapLoop()
	
	# Public info
	es.set("et_mod_version", info.version, "ETmod version")
	es.makepublic("et_mod_version")
	
	# Log
	log("Mod loaded")
''' Script unloaded '''
def unload():
	# Unload sub-scripts
	unloadSubScripts()
	
	# Save players stats
	for p in players.values():
		if	not p.IsBot() and\
			not p.IsUndefined() and\
			not p.IsPending():
			log("Saving " + p.Name + "'s stats")
			p.SaveStats()
			p.SavePreferences()
			p.SaveSkills()
			
	# Unregister everything that was registered for the mod
	unregSayHooks()
	unregClientCmds()
	unregServerCmds()
	
	# Reset the variable used for the loop to stop it
	es.set("et_mod_loaded", 0)
	
	# Log
	log("Mod unloaded")
''' New map started '''
def es_map_start(event_var):
	# Trying to get them to fire everywhere with a little delay...
	gamethread.delayed(0, customMapChange, (event_var))

#___________________________________________SPECIAL EVENTS

# ---------------------- #
# SPECIAL EVENTS #
# ---------------------- #
def customMapChange(event_var):
	# Log
	log("Map started: " + event_var["mapname"])
	
	# Prepare the server
	getModReady(event_var["mapname"])
def customRoundStart(event_var):
	# Log
	log("Round started")
	
	# Advert
	es.msg("Type 'etmenu' to open the main menu")
	
	# Reset special stuff
	for p in players.values():
		p.Situation.Planted = 0
		
	# Reset server config if map has just changed
	if	str(server_var["et_eventscripts_previousmap"]) == "" or\
		str(server_var["et_eventscripts_previousmap"]) != str(server_var["eventscripts_currentmap"]):
		getModReady(server_var["eventscripts_currentmap"])
	
#___________________________________________PLAYER EVENTS

# --------------------- #
# PLAYER EVENTS #
# --------------------- #
''' Player joining server '''
def player_connect(event_var):
	# Log
	log("Player connected: " + event_var["name"] + " (" + event_var["networkid"] + ")")
	
	# Create player in-game info
	if not players.has_key(event_var["userid"]):
		players[event_var["userid"]] = Player()
	p = players[event_var["userid"]]
	p.Index = event_var["index"]
	p.ID = event_var["userid"]
	p.SteamID = event_var["networkid"]
	p.Name = event_var["name"]
	
	# Load stats and preferences if required
	p.LoadStats()
	p.LoadPreferences()
	p.LoadSkills()
	
''' Player successfully joined server '''
def player_activate(event_var):
	# Only if userid is not 0
	if int(event_var["userid"]) != 0:
		# Create temp info if needed
		if not players.has_key(event_var["userid"]):
			players[event_var["userid"]] = Player()
		p = players[event_var["userid"]]
		p.ID = event_var["userid"]
		p.SteamID = event_var["es_steamid"]
		
		# Prepare situation
		p.Situation.Alive = False
		p.Situation.InvAmount = 0
		
		# Save stats if human with already a valid steamid
		if	not p.IsUndefined() and\
			not p.IsPending() and\
			not p.IsBot():
			p.LoadStats()
			p.LoadPreferences()
			p.LoadSkills()
		
''' Player disconnecting from server '''
def player_disconnect(event_var):
	# Log
	log("Player disconnecting: " + str(event_var["es_username"]))
	
	# Save stats if human with already a valid steamid
	if players.has_key(event_var["userid"]):
		p = players[event_var["userid"]]
		if	not p.IsUndefined() and\
			not p.IsPending() and\
			not p.IsBot():
			p.SaveStats()
			p.SavePreferences()
			p.SaveSkills()
			
		# Remove player temp info
		players.pop(event_var["userid"])		
	
''' Player changed team '''
def player_team(event_var):
	# Log
	log("Player " + str(event_var["es_username"]) + " chose team " + str(event_var["team"]))
	
	# Update player info
	if players.has_key(event_var["userid"]):
		players[event_var["userid"]].Team = int(event_var["team"])
		
''' Player chose a game class - Not working anymore ?'''
def player_class(event_var):
	# Respawn player immediately
	if players.has_key(event_var["userid"]):	# player's temp info exists
		p = players[event_var["userid"]]
		if p.Team > 1:							# T or CT
			es.delayed(2, "est_Spawn " + str(event_var["userid"]) + " 1")
			
''' A player died '''
def player_death(event_var):
	# Remove cash automatically given to attacker in order for him not to be able to buy stuff
	es.server.queuecmd("est_SetCash " + str(event_var["userid"]) + " 0")
	es.server.queuecmd("est_SetCash " + str(event_var["attacker"]) + " 0")
	
	# Retreive players info
	victim = None
	if players.has_key(event_var["userid"]):
		victim = players[event_var["userid"]]
	attacker = None
	if players.has_key(event_var["attacker"]):
		attacker = players[event_var["attacker"]]
		
	# Check for suicide
	if event_var["userid"] == event_var["attacker"]:
		if victim is not None:
			log(victim.Name + " committed suicide")
			victim.Stats.Update(victim.Class.Name, victim.Class.Name, "suicides", victim.Stats.GetLevel(victim.Class.Name))
		else:
			log("Suicide skipped for " + victim.Name)
	# Check for team-kill
	elif event_var["es_userteam"] == event_var["es_attackerteam"]:
		if victim is not nNone and attacker is not None:
			log(attacker.Name + " team-killed " + victim.Name)
			attacker.Stats.Update(attacker.Class.Name, victim.Class.Name, "teamkillsdone", victim.GetLevel(victim.Class.Name))
			victim.Stats.Update(attacker.Class.Name, victim.Class.Name, "teamkills", attacker.GetLevel(attacker.Class.Name))
			# Headshot bonus
			if event_var["headshot"]:
				attacker.Stats.Update(attacker.Class.Name, victim.Class.Name, "headshots")
		else:
			log("Team-kill skipped for " + str(es.getplayername(event_var["attacker"])) +\
				" on " + str(es.getplayername(event_var["userid"])))
	# Check for enemy kill
	else:
		if attacker is not None and victim is not None:
			log(attacker.Name + " killed " + es.getplayername(event_var["userid"]))
			attacker.Stats.Update(attacker.Class.Name, victim.Class.Name, "kills", victim.Stats.GetLevel(victim.Class.Name))
			victim.Stats.Update(attacker.Class.Name, victim.Class.Name, "deaths", attacker.Stats.GetLevel(attacker.Class.Name))
		else:
			log("Kill skipped for " + str(es.getplayername(event_var["attacker"])) +\
				" on " + str(es.getplayername(event_var["userid"])))
				
	# Update victim situation
	if victim is not None:
		victim.Situation.Alive = False
		
	# Show little info to both
	displayInGameInfo(attacker)
	displayInGameInfo(victim)
	
	# If 'et_synchronized_spawn' is set to 0, respawn the player after the spawn delay
	if es.getInt("et_synchronized_spawn") != 1:
		es.delayed(es.getInt("et_spawn_delay"), "est_RemoveIdle cs_ragdoll")
		es.delayed(es.getInt("et_spawn_delay"), "est_RemoveIdle item")
		es.delayed(es.getInt("et_spawn_delay"), "est_RemoveIdle weapon")
		es.delayed(es.getInt("et_spawn_delay"), "est_Spawn " + str(event_var["userid"]) + " 1")
	
''' A player was injured '''
def player_hurt(event_var):
	# Remove cash automatically given to attacker in order for him not to be able to buy stuff
	es.server.queuecmd("est_SetCash " + str(event_var["userid"]) + " 0")
	es.server.queuecmd("est_SetCash " + str(event_var["attacker"]) + " 0")
	
	# Retreive players info
	victim = None
	if players.has_key(event_var["userid"]):
		victim = players[event_var["userid"]]
	attacker = None
	if players.has_key(event_var["attacker"]):
		attacker = players[event_var["attacker"]]
		
	# Check for self-hurt
	if event_var["userid"] == event_var["attacker"]:
		if victim is not None:
			log(victim.Name + " injured himself")
			victim.Stats.Update(victim.Class.Name, victim.Class.Name, "selfhurts", victim.Stats.GetLevel(victim.Class.Name))
		else:
			log("Self hurt skipped for " + victim.Name)
	# Check for team-hurt
	elif event_var["es_userteam"] == event_var["es_attackerteam"]:
		if victim is not None and attacker is not None:
			log(attacker.Name + " has team-hurt " + victim.Name)
			attacker.Stats.Update(attacker.Class.Name, victim.Class.Name, "teamhurtsdone", victim.Stats.GetLevel(victim.Class.Name))
			victim.Stats.Update(attacker.Class.Name, victim.Class.Name, "teamhurts", attacker.Stats.GetLevel(attacker.Class.Name))
		else:
			log("Team-hurt skipped for " + str(es.getplayername(event_var["attacker"])) +\
				" on " + str(es.getplayername(event_var["userid"])))
	# Check for enemy hurt
	else:
		if attacker is not None:
			log(attacker.Name + " has hurt " + es.getplayername(event_var["userid"]))
			attacker.Stats.Update(attacker.Class.Name, victim.Class.Name, "hurtsdone", victim.Stats.GetLevel(victim.Class.Name))
			victim.Stats.Update(attacker.Class.Name, victim.Class.Name, "hurts", attacker.Stats.GetLevel(attacker.Class.Name))
			
			# Check if victim should lose something due to skills
			victim.SkillsDamage(attacker)
		else:
			log("Hurt skipped for " + str(es.getplayername(event_var["attacker"])) +\
				" on " + str(es.getplayername(event_var["userid"])))
		
	# Show little info to both
	displayInGameInfo(attacker)
	displayInGameInfo(victim)
	
''' Player respawning '''
def player_spawn(event_var):
	# Remove his cash
	es.server.queuecmd("est_SetCash " + str(event_var["userid"]) + " 0")
	
	# Get player instance
	p = players.get(event_var["userid"], None)
	
	# If player info does not yet exist, whatever the reason, whichever team he's on, create them
	if p is None and int(event_var["userid"]) != 0:
		p = Player()
		players[event_var["userid"]] = p
		p.ID = event_var["userid"]
		p.SteamID = event_var["es_steamid"]
		p.Situation.InvAmount = 0
	
	# Update his situation - check team because this one is triggered even for teams 0 and 1 (unassigned and spec)
	if p is not None:
		p.Situation.Alive = (p.Team > 1)
		# Only do something if team is T or CT
		if p.Team > 1:
			# Give a random class to bots and to players if 'et_random_classes' is set to 1
			if p.IsBot() or es.getInt("et_random_classes") == 1:
				checked = []
				while(1):
					c = getRandomClass()
					# Add class to checked ones if not already into the list
					if c not in checked:
						checked.append(c)
					# Check if that class is not full and continue if it is
					if isClassNotFull(c, p.Team) and willClassNotBeFull(c, p.Team):
						# If player is not bot, check that he can join that class
						if not p.IsBot():
							joinClass(p, c)
						# If bot, just set his class
						else:
							p.Class = c
						break
					#If all classes have been checked, give him that one anyway
					elif len(checked) == len(classes) - 1:	# don't count 'noclass' as a class
						p.Class = c
						break
			# If player already chose a class let him join it
			elif p.NextClass is not None:
				p.Class = p.NextClass
			# Otherwise, show him the class selection menu if he doesn't yet have a class
			elif p.Class == classes["noclass"]:
				# Only if not already displayed
				if not popuplib.isqueued("etMenuChooseClass", p.ID):
					showMenuChooseClass(p.ID)
		# Show the menu in 'unassigned' or 'spectator' teams in cas of a forced respawn
		#elif not p.IsBot():
			# Only if not already displayed
		#	if not popuplib.isqueued("etMenuChooseClass", p.ID):
		#		showMenuChooseClass(p.ID)
	
	# Give stuff if in a playing team
	if p is not None and p.Team > 1:
		p.GiveClassStuff(True)	# in god mode at first
		es.tell(p.ID, "Spawn protection ON")
		spt = es.getInt("et_spawn_protection_time")
		es.delayed(spt, "est_God " + p.ID + " 0;es_tell " + str(p.ID) + " Spawn protection OFF")
	
''' Player changed his name '''
def player_changename(event_var):		
	# Update player info
	if players.has_key(event_var["userid"]):
		players[event_var["userid"]].Name = event_var["newname"]
	
''' Player using a weapon'''
def weapon_fire(event_var):
	# Get player info
	p = players.get(event_var["userid"], None)
	
	# Check if player info exist
	if p is not None:
		l = p.Preferences.Language
		
		# Check if weapon used was the inventory weapon
		if	(p.Team == 2 and p.Class.InventoryT == weapons[event_var["weapon"]]) or\
			(p.Team == 3 and p.Class.InventoryCT == weapons[event_var["weapon"]]):
			
			# Decrease remaining count
			p.Situation.InvAmount -= 1
			
			# Give another one if allowed
			if p.Situation.InvAmount > 0:
				if p.Team == 2:
					es.server.queuecmd("est_Give " + event_var["userid"] + " " + p.Class.InventoryT.FullName())
				else:
					es.server.queuecmd("est_Give " + event_var["userid"] + " " + p.Class.InventoryCT.FullName())
				es.tell(event_var["userid"], l.Translate("Remaining " + event_var["weapon"] + ": " + str(p.Situation.InvAmount)))
			else:
				es.tell(event_var["userid"], l.Translate("No " + event_var["weapon"] + " left"))
				
		# Check if player's clip should be automatically reloaded / unlimited ammo should be used
		elif es.getInt("et_no_reload") or es.getInt("et_unlimited_ammo") == 1:
		
			# Unlimited ammo ?
			unlAmmo = es.getInt("et_unlimited_ammo")
			
			# Check for Ts
			if p.Team == 2:	
			
				# Check for primary weapon
				if p.Class.WeaponPrimaryT == weapons[event_var["weapon"]]:
				
					# Get the weapon instance
					weapon = p.Class.WeaponPrimaryT
					
					# Get remaining ammo in pack
					ammo = es.getplayerprop(p.ID, propIndices["ammo"] + weapon.AmmoType.Index)
					
					# Decrease ammo pack if 'et_unlimited_ammo' is not activated					
					if unlAmmo != 1:
						weapon.SetAmmo(p.ID, ammo - 1)
					
					# Give only if ammo pack is not emty or if 'et_unlimited_ammo' is set to 1
					if unlAmmo == 1 or ammo > 0:
						weapon.SetClip(p.ID, p.Class.ClipPrimaryT)
						
				# Check for secondary weapon
				elif p.Class.WeaponSecondaryT == weapons[event_var["weapon"]]:
				
					# Get the weapon instance
					weapon = p.Class.WeaponSecondaryT
					
					# Get remaining ammo in pack
					ammo = es.getplayerprop(p.ID, propIndices["ammo"] + weapon.AmmoType.Index)
					
					# Decrease ammo pack if 'et_unlimited_ammo' is not activated
					if unlAmmo != 1:
						weapon.SetAmmo(p.ID, ammo - 1)
					
					# Give only if ammo pack is not emty or if 'et_unlimited_ammo' is set to 1
					if unlAmmo == 1 or ammo > 0:
						weapon.SetClip(p.ID, p.Class.ClipSecondaryT)
						
			# Check for CTs
			elif p.Team == 3:
			
				# Check for primary weapon
				if p.Class.WeaponPrimaryCT == weapons[event_var["weapon"]]:
				
					# Get the weapon instance
					weapon = p.Class.WeaponPrimaryCT
					
					# Get remaining ammo in pack
					ammo = es.getplayerprop(p.ID, propIndices["ammo"] + weapon.AmmoType.Index)
					
					# Decrease ammo pack if 'et_unlimited_ammo' is not activated
					if unlAmmo != 1:
						weapon.SetAmmo(p.ID, ammo - 1)
					
					# Give only if ammo pack is not emty or if 'et_unlimited_ammo' is set to 1
					if unlAmmo == 1 or ammo > 0:
						weapon.SetClip(p.ID, p.Class.ClipPrimaryCT)
						
				# Check for secondary weapon
				elif p.Class.WeaponSecondaryCT == weapons[event_var["weapon"]]:
					# Get the weapon instance
					weapon = p.Class.WeaponSecondaryCT
					
					# Get remaining ammo in pack
					ammo = es.getplayerprop(p.ID, propIndices["ammo"] + weapon.AmmoType.Index)
					
					# Decrease ammo pack if 'et_unlimited_ammo' is not activated
					if unlAmmo != 1:
						weapon.SetAmmo(p.ID, ammo - 1)
					
					# Give only if ammo pack is not emty or if 'et_unlimited_ammo' et set to 1
					if unlAmmo == 1 or ammo > 0:
						weapon.SetClip(p.ID, p.Class.ClipSecondaryCT)			
	else:
		es.tell(event_var["userid"], "Player info not found")
		
''' Player picked up something '''
def item_pickup(event_var):
	# Get player info
	p = players.get(event_var["userid"], None)
	
	# Drop if he has no info or his class is not supposed to use that
	if	p is None or\
		(event_var["item"] is not None and\
		event_var["item"] != "" and\
		event_var["item"] != "knife" and\
		p.Class.WeaponPrimaryT != weapons[event_var["item"]] and\
		p.Class.WeaponSecondaryT != weapons[event_var["item"]] and\
		p.Class.InventoryT != weapons[event_var["item"]] and\
		p.Class.WeaponPrimaryCT != weapons[event_var["item"]] and\
		p.Class.WeaponSecondaryCT != weapons[event_var["item"]] and\
		p.Class.InventoryCT != weapons[event_var["item"]]):
		#es.server.queuecmd("est_RemoveWeapon " + str(p.ID) + " " + "weapon_" + event_var["item"])
		es.server.queuecmd("est_DropWeapon " + str(p.ID) + " " + "weapon_" + event_var["item"])
		l = p.Preferences.Language
		es.tell(p.ID, l.Translate("You're not authorized to use weapon " + event_var["item"] +" in class " + p.Class.Name))
	
#________________________________________________________________GAME EVENTS

# ------------------- #
# GAME EVENTS #
# ------------------- #
''' A new round started '''
def round_start(event_var):
	# Trying to get them to fire everywhere with a little delay...
	gamethread.delayed(0, customRoundStart, (event_var))
	
''' A round ended '''
def round_end(event_var):
	# Log
	log("Round ended")
	
	# If there's only one player on server, no need to give him round xp
	if len(players) < 2:
		return
	
	# Give round Xp to all players of the team who won the round
	for p in players.values():
		if not p.IsBot() and p.Team == int(event_var["winner"]):
			p.Stats.Update(p.Class, p.Class, "roundswon")
			l = p.Preferences.Language
			es.tell(p.ID, l.Translate("You won '" + str(es.getInt("et_round_xp")) + "xp' for winning this round"))
''' A bomb is being planted '''
def bomb_beginplant(event_var):
	# Get player info
	p = players.get(event_var["userid"], None)
	
	# Prevent from planting if possible
	if p is None or not p.Class.Actions.CanPlant():
		preventFromPlanting(p.ID)
		
''' A bomb is being defused '''
def bomb_begindefuse(event_var):
	# Get player info
	p = players.get(event_var["userid"], None)
	
	# Prevent from planting if possible
	if p is None or not p.Class.Actions.CanDefuse():
		preventFromDefusing(p.ID)
	
''' A bomb has been planted '''
def bomb_planted(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Bomb planted by " + p.Name)
	
	# Update stats if ok
	if	p is not None and\
		p.Team > 1 and\
		p.Class.Actions.CanPlant():
		p.Stats.Update(p.Class.Name, p.Class.Name, "plants")
		
		# Decrease remaining count
		p.Situation.InvAmount -= 1
		
		# Update planted count
		p.Situation.Planted = p.Situation.Planted + 1
		
		# Show little info
		displayInGameInfo(p)
		
		# Give another one if allowed and decrease remaining count
		l = p.Preferences.Language
		if p.Situation.InvAmount > 0:
			es.server.queuecmd("est_Give " + event_var["userid"] + " weapon_c4")
			es.tell(event_var["userid"], l.Translate("Remaining bombs: " + str(p.Situation.InvAmount)))
		else:
			es.tell(event_var["userid"], l.Translate("No bomb remaining"))
	else:
		log("Skipped bomb planted actions for " + str(es.getplayername(event_var["userid"])))
	
''' A bomb has exploded '''
def bomb_exploded(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Bomb exploded by " + p.Name)
	
	# Update stats if ok
	if	p is not None and\
		p.Team > 1 and\
		p.Class.Actions.CanExplode():
		
		# Update stats
		p.Stats.Update(p.Class.Name, p.Class.Name, "explodes")
		
		# Show little info
		displayInGameInfo(p)
		
		# Update planted count
		p.Situation.Planted = p.Situation.Planted - 1
	else:
		log("Skipped bomb exploded actions for " + str(es.getplayername(event_var["userid"])))
''' A bomb has been defused '''
def bomb_defused(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Bomb defused by " + p.Name)
	
	# Update stats if ok
	if	p is not None and\
		p.Team > 1 and\
		p.Class.Actions.CanDefuse():
		
		# Update stats
		p.Stats.Update(p.Class.Name, p.Class.Name, "defuses")
		
		# Show little info
		displayInGameInfo(p)
	else:
		log("Skipped bomb defused actions for " + str(es.getplayername(event_var["userid"])))	
''' A hostage has been rescued '''
def hostage_rescued(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Hostage rescued by " + p.Name)
	
	# Fire event if ok
	if	p is not None and\
		p.Team > 1 and\
		p.Class.Actions.CanRescue():
		
		# Update stats
		p.Stats.Update(p.Class.Name, p.Class.Name, "rescues")
		
		# Show little info
		displayInGameInfo(p)
	else:
		log("Skipped hostage rescue actions for " + str(es.getplayername(event_var["userid"])))		
''' A hostage has been hurt '''
def hostage_hurt(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Hostage hurt by " + p.Name)
	
	# Fire event if ok
	if	p is not None and\
		p.Team > 1:
		p.Stats.Update(p.Class.Name, p.Class.Name, "hosthurts")
		displayInGameInfo(p)
	else:
		log("Skipped hostage hurt actions for " + str(es.getplayername(event_var["userid"])))		
''' A hostage has been killed '''
def hostage_killed(event_var):
	# Get user info
	p = players.get(event_var["userid"], None)
	
	# Log
	if p is not None:
		log("Hostage killed by " + p.Name)
	
	# Fire event if ok
	if	p is not None and\
		p.Team > 1:
		p.Stats.Update(p.Class.Name, p.Class.Name, "hostkills")
		displayInGameInfo(p)
	else:
		log("Skipped hostage killed actions for " + str(es.getplayername(event_var["userid"])))
		
#___________________________________________ES_TOOLS EVENTS

# ------------------------- #
# ES_TOOLS EVENTS #
# ------------------------- #
''' Hooked text used '''
def est_sayhook(event_var):
	# Get player info
	p = players.get(event_var["userid"], None)
	
	# If player has info
	if p is not None:
		# Get language
		l = p.Preferences.Language
		
		# Display ETmod menu
		if event_var["hooktext"] == "etmenu":
			# No commandstring = main menu
			if event_var["commandstring"] == "":
				showMenuMain(p.ID)
			# Class
			elif event_var["commandstring"] == "class":
				showMenuChooseClass(p.ID)
			# Stats
			elif event_var["commandstring"] == "stats":
				showMenuStats(p.ID)
			# Preferences
			elif event_var["commandstring"] == "pref":
				showMenuPreferences(p.ID)
		
		# Respawn
		elif event_var["hooktext"] == "etspawn":
			es.server.queuecmd("est_CExec " + str(event_var["userid"]) + " et_spawn")
				
		# Choose class
		elif event_var["hooktext"] == "etclass":
			# Check if chosen class exists
			if classes.has_key(event_var["commandstring"]):
				c = classes[event_var["commandstring"]]
				joinClass(p, c)
			# Otherwise tell him that the chosen class doesn't exist
			else:
				es.tell(p.ID, l.Translate("That class doesn't exist"))
		# Check level
		elif event_var["hooktext"] == "etlevel":
			# If no class requested, check for current class
			c = ""
			if event_var["commandstring"] == "":
				c = p.Class.Name
			# If a class was requested, get that one
			elif classes.has_key(event_var["commandstring"]):
				c = event_var["commandstring"]
			# If the requested class does not exist, tell him and return
			else:
				es.tell(event_var["userid"], "The requested class does not exist")
				
			# Tell him about his level and xp in that class
			es.tell(event_var["userid"], l.Translate("Your level in class " +\
				c + " is currently " + str(p.Stats.GetLevel(c)) +\
				" (" + str(p.Stats.Classes[c]["xp"]) + "xp)"))
		# Heal
		elif event_var["hooktext"] == "etheal":
			es.server.queuecmd("est_CExec " + str(event_var["userid"]) + " et_heal")
		# Supply ammo
		elif event_var["hooktext"] == "etsupply":
			es.server.queuecmd("est_CExec " + str(event_var["userid"]) + " et_supply")
		# use skill
		elif event_var["hooktext"] == "etskill":
			es.server.queuecmd("est_CExec " + str(event_var["userid"]) + " et_skill " + event_var["commandstring"])
	# Otherwise tell him
	else:
		es.tell(event_var["userid"], "Info not found")
	
#________________________________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #	
''' Load sub-scripts '''
def loadSubScripts():
	es.load("etmod_2008/includes")
	es.load("etmod_2008/languages")
	es.load("etmod_2008/logs")
	es.load("etmod_2008/classes")
	es.load("etmod_2008/skills")
	es.load("etmod_2008/stats")
	es.load("etmod_2008/players")
	es.load("etmod_2008/admins")
	es.load("etmod_2008/shop")
	es.load("etmod_2008/menus")
	es.load("etmod_2008/server")
	log("Sub-scripts loaded")
''' Unload sub-scripts '''
def unloadSubScripts():
	es.unload("etmod_2008/server")
	es.unload("etmod_2008/menus")
	es.unload("etmod_2008/shop")
	es.unload("etmod_2008/admins")
	es.unload("etmod_2008/players")
	es.unload("etmod_2008/stats")
	es.unload("etmod_2008/skills")
	es.unload("etmod_2008/classes")
	es.unload("etmod_2008/logs")
	es.unload("etmod_2008/languages")
	es.unload("etmod_2008/includes")
	log("Sub-scripts unloaded")
''' Prepare the mod variables '''
def prepareVariables():
	es.set("et_restrict_classes", 1, "Use restrictions defined in class config files")
	es.set("et_Restrict_players_in_classes", 1, "Use percent allowed defined in class config files")
	es.set("et_unlimited_ammo", 0, "Give unlimited ammo to players")
	es.set("et_no_reload", 0, "Allow players not to have to reload when their clip is empty")
	es.set("et_spawn_delay", 5, "Delay before respawning after dying")
	es.set("et_spawn_protection_time", 4, "Time during which players can't be hurt after respawning")
	es.set("et_synchronized_spawn", 0, "All dead players respawn will respawn together")
	es.set("et_random_classes", 0, "Players will be given a random class at spawn like for bots")
	es.set("et_use_colors", 0, "Use colors defined in class config files to see what class players are in")
	es.set("et_round_xp", 50, "Xp given to players winning a round")
	es.set("et_round_money", 500, "Money given to players winning a round")
	es.set("et_hurt_loose_xp", 0, "Whether players getting hurt should loose xp or not")
	es.set("et_noconfig_map_behaviour", "keep", "The behaviour to adopt on map change if there's no config file for the map." +\
		"\n\t- keep: keep previous config\n\t- reset: use default config")
''' Register say hooks '''
def regSayHooks():
	es.server.queuecmd("est_RegSayHook etspawn 0")
	es.server.queuecmd("est_RegSayHook etclass 1")
	es.server.queuecmd("est_RegSayHook etlevel 1")
	es.server.queuecmd("est_RegSayHook etheal 0")
	es.server.queuecmd("est_RegSayHook etsupply 0")
	es.server.queuecmd("est_RegSayHook etmenu 1")
	es.server.queuecmd("est_RegSayHook etskill 1")
	log("Say hooks registered")
''' Register client commands '''
def regClientCmds():
	es.regclientcmd("et_rcon", "etmod_2008/et_rcon", "Execute a server command")
	es.regclientcmd("et_heal", "etmod_2008/et_heal", "Heal someone")
	es.regclientcmd("et_supply", "etmod_2008/et_supply", "Supply ammo to someone")
	es.regclientcmd("et_class", "etmod_2008/et_class", "Join a class")
	es.regclientcmd("et_menu", "etmod_2008/et_menu", "Open the main menu or the specified one")
	es.regclientcmd("et_skill", "etmod_2008/et_skill", "Use a skill")
	es.regclientcmd("et_spawn", "etmod_2008/et_spawn", "Respawn if allowed")
	log("Client commadns registered")
''' Register custom server commands '''
def regServerCmds():
	es.regcmd("et_players", "etmod_2008/et_players", "Displays a list of the players with some info")	
	es.regcmd("et_classes", "etmod_2008/et_classes", "Displays info about the mod classes")
	es.regcmd("et_skills", "etmod_2008/et_skills", "Displays info about the available skills")
	es.regcmd("et_shutdown", "etmod_2008/et_shutdown", "Shutdown the server after saving all stats")
	es.regcmd("et_saveall", "etmod_2008/et_saveall", "Save players stats, preferences and skills")
	log("Server commands registered")
''' Unregister say hooks '''
def unregSayHooks():
	es.server.queuecmd("est_UnRegSayHook etmenu")
	es.server.queuecmd("est_UnRegSayHook etspawn")
	es.server.queuecmd("est_UnRegSayHook etclass")
	es.server.queuecmd("est_UnRegSayHook etlevel")
	es.server.queuecmd("est_UnRegSayHook etheal")
	es.server.queuecmd("est_UnRegSayHook etsupply")
	es.server.queuecmd("est_UnRegSayHook etlook")
	es.server.queuecmd("est_UnRegSayHook etskill")
	log("Say hooks unregistered")
''' Unregister client commands '''
def unregClientCmds():
	es.unregclientcmd("et_rcon")
	es.unregclientcmd("et_heal")
	es.unregclientcmd("et_supply")
	es.unregclientcmd("et_menu")
	es.unregclientcmd("et_skill")
	log("Client commands unregistered")
''' Unregister server commands '''
def unregServerCmds():
	try:
		es.unregcmd("et_players")
		es.unregcmd("et_classes")
		es.unregcmd("et_skills")
		es.unregcmd("et_shutdown")
		es.unregcmd("et_saveall")
		log("Server commands unregistered")
	except:
		log("es.unregcmd not working")
''' Prepare the server and mod config '''
def getModReady(mapname):
	# Search for a server config for this map
	config = ServerConfig("tmp")
	config.LoadFromFile(mapname)
	config.Use()
		
	# Search for a mod config for this map
	config = ModConfig("tmp")
	config.LoadFromFile(mapname)
	config.Use()
	
	# In case 'es_map_start' would not fire
	es.set("et_eventscripts_previousmap", str(server_var["eventscripts_currentmap"]))
	
	# Restart game to make sure all server config will be in use
	es.server.queuecmd("mp_restartgame 5")
''' Some player info displayed for kills and the like '''
def displayInGameInfo(player):
	if player is None or player.IsBot() or not player.Preferences.InGameInfo:
		return
	#showMenuQuickStats(player.ID)
	es.menu(5, player.ID, "[ETmod2008 by McFly] Quick stats\n---------------------------------------\n" +\
		" \nMoney: " + str(player.Stats.Money) + "\n" +\
		" \nSpecific " + player.Class.Name + " class info:\n" +\
		"  Level: " + str(player.Stats.GetLevel(player.Class.Name)) + "\n" +\
		"  Xp: " + str(player.Stats.Classes[player.Class.Name]["xp"]))
''' A loop to let people know when they will respawn and respawn them '''
def expansiveLoop(elapsed):
	# Stop the loop if mod is no more loaded
	if es.getInt("et_mod_loaded") == 0:
		return
	
	# Calculate elapsed time
	delay = es.getInt("et_spawn_delay")
	elapsed += 1
	
	# If respawn time, respawn dead players	
	if elapsed > delay:
		elapsed = 0
		
	# Tell dead players and respawn them when needed if 'et_synchronized_spawn' is on
	if es.getInt("et_synchronized_spawn") == 1:
		if elapsed == 0:
			es.server.queuecmd("est_Spawn #ct!l 1")
			# Remove idle stuff after players respawned and were given their right stuff
			es.delayed(1, "est_RemoveIdle cs_ragdoll")
			es.delayed(1, "est_RemoveIdle item")
			es.delayed(1, "est_RemoveIdle weapon")
		for p in players.values():
			if not p.Situation.Alive and not p.IsBot():
				es.centertell(p.ID, "Respawn in " + str(delay - elapsed) + " seconds")
	
	# Loop
	gamethread.delayed(1, expansiveLoop, (elapsed))
''' Same loop but executed only at every respawn time to reduce its cost '''
''' This means that centered messaged can't be displayed this way '''
def cheapLoop():
	# if the mod has been unloaded, stop the loop
	if es.getInt("et_mod_loaded") == 0:
		return
		
	# Respawn dead players if 'et_synchronized_spawn' is on
	if es.getInt("et_synchronized_spawn") == 1:
		es.server.queuecmd("est_Spawn #ct!d 1")
		# Remove idle stuff after players respawned and were given their right stuff
		es.delayed(1, "est_RemoveIdle cs_ragdoll")
		es.delayed(1, "est_RemoveIdle item")
		es.delayed(1, "est_RemoveIdle weapon")
	
	#Loop
	gamethread.delayed(es.getInt("et_spawn_delay"), cheapLoop, ())
	
#________________________________________________________________CLIENT COMMANDS

# -------------------------- #
# CLIENT COMMANDS #
# -------------------------- #
''' Execute an rcon command '''
def et_rcon():
	#Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info and language
	p = players.get(id, None)
	
	# No arguments = show syntax
	if es.getargc() == 1:
		if p is not None:
			l = p.Preferences.Language
			es.tell(id, l.Translate("Syntax: et_rcon <server command>"))
			es.tell(id, l.Translate("PS: Note that you need to be and admin to use this"))
		else:
			es.tell(id, "Syntax: et_rcon <server command>")
			es.tell(id, "PS: Note that you need to be an admin to use this")
	else:
		# Otherwise execute command if player is admin
		if p is None:
			es.tell(id, "Error: player info not found")
		else:
			es.server.cmd(es.getargv(1))
			
''' Respawn if not yet in a team and not playing '''
def et_spawn():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info
	p = players.get(id, None)
	
	# If player info not found, skill for now (too tired...)
	if p is None: return
	
	# Get player language
	l = p.Preferences.Language
	
	# In a playing team
	if p.Team > 1:
		# If 'et_synchronized_spawn' is set to 1, player will auto-respawn later so tell him
		if es.getInt("et_synchronized_spawn") == 1:
			es.tell(p.ID, l.Translate("You'll respawn with others in a moment"))
		# Otherwise spawn him if he's dead
		else:
			# Dead
			if not p.Situation.Alive:
				es.server.queuecmd("est_Spawn " + str(p.ID) + " 1")
			# Alive
			else:
				es.tell(p.ID, l.Translate("You cannot respawn whenever you want"))
	# Not in a playing team
	else:
		es.tell(p.ID, l.Translate("Choose a team first"))
			
''' Heal someone '''
def et_heal():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info
	p = players.get(id, None)
	
	# If his class is not allowed to heal, warn him
	if p is None:
		es.tell(id, "Players without a class cannot heal others")
		return
	# If his clas cannot heal, tell him
	elif not p.Class.Actions.CanHeal():
		l = p.Preferences.Language
		es.tell(id, l.Translate("Players in class " + p.Class.Name + " cannot heal others"))
		return
	# If he's dead, don't allow him to heal others
	elif not p.Situation.Alive:
		l = p.Preferences.Language
		es.tell(id, l.Translate("Dead players cannot heal others"))
		return
	
	# Get the player he's looking at if there's one
	es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(id) + " et_view_player")
	
	# If there's none, heal him instead
	if int(server_var["et_view_player"]) == 0:
		p.Heal(p)
	# If there's one, check if it's a teammate and heal him
	elif players[es.getString("et_view_player")].Team == players[id].Team:
		if players[es.getString("et_view_player")].Heal(p):
			# Update stats
			p.Stats.Update(p.Class.Name, p.Class.Name, "heals")
			# Show little info
			displayInGameInfo(p)
	# Otherwise heal him
	else:
		p.Heal()
	
''' Supply ammo to someone '''
def et_supply():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info
	p = players.get(id, None)
	
	# If his class is not allowed to supply ammo, warn him
	if p is None:
		es.tell(id, "Players without a class cannot supply ammo")
		return
	# If his class cannot supply ammo, tell him
	elif not p.Class.Actions.CanSupply():
		l = p.Preferences.Language
		es.tell(id, l.Translate("Players in class " + p.Class.Name + " cannot supply ammo"))
		return
	# If he's dead, don't allow him to supply ammo
	elif not p.Situation.Alive:
		l = p.Preferences.Language
		es.tell(id, l.Translate("Dead players cannot supply ammo"))
		return
		
	# Get the player he's looking at if there's one
	es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(id) + " et_view_player")
	
	# If there's none, give ammo to him instead
	if int(server_var["et_view_player"]) == 0:
		p.Supply(p)
	# If there's one, check if it's a teammate and give him ammo
	elif players[es.getString("et_view_player")].Team == players[id].Team:
		if players[es.getString("et_view_player")].Supply(p):
			# Update stats
			p.Stats.Update(p.Class.Name, p.Class.Name, "supplies")
			# Show little info
			displayInGameInfo(p)
	# Otherwise give it to him
	else:
		p.Supply()
		

''' Join a class '''
def et_class():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Which class ?
	cName = es.getargv(1)
	
	# Check if that class exists
	if cName in classes.keys():
		# Make player join the class
		joinClass(players[id], classes[cName])
	# Otherwise tell him
	else:
		l = players[id].Preferences.Language
		es.tell(id, l.Translate("That class doesn't exist"))

''' open the main menu '''
def et_menu():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info
	p = players.get(id, None)
	
	# If player info exists, show him the menu
	if p is not None:
		# If no argument, show main menu
		if es.getargc() == 1:
			showMenuMain(p.ID)
		# Otherwise get and check argument
		else:
			arg = es.getargv(1)
			# Class
			if arg == "class":
				showMenuChooseClass(p.ID)
			# Stats
			elif arg == "stats":
				showMenuStats(p.ID)
			# Preferences
			elif arg == "pref" or arg == "preferences":
				showMenuPreferences(p.ID)
	else:
		es.tell(id, "Info not found")
		
''' Use a skill '''
def et_skill():
	# Who's concerned ?
	id = str(es.getcmduserid())
	
	# Get player info
	p = players.get(id, None)
	
	# If player info exists, show him the menu
	if p is not None:
		# If no argument, show player the menu with his skills
		if es.getargc() == 1:
			# If he's dead, tell him dead players cannot use skills
			if not p.Situation.Alive:
				es.tell(p.ID, "Dead players cannot use skills")
			# Otherwise show him the skill selection menu
			else:
				showMenuChooseSkill(p.ID)
		# Otherwise check what skill was precised
		else:
			skill = es.getargv(1)
			# If that skill does not exist, tell him
			if not skills.has_key(skill):
				es.tell(p.ID, "The '" + skill + "' skill does not exist")
			# If he hasn't learnt it yet, tell him
			elif not p.Skills.has_key(skill):
				es.tell(p.ID, "You haven't learnt that skill yet.")
			# If he's dead, tell him he can't use skills
			elif not p.Situation.Alive:
				es.tell(p.ID, "Dead players cannot use skills")
			# Otherwise, try to use it
			else:
				p.Skills[skill].Use()
#________________________________________________________________SERVER COMMANDS

# -------------------------- #
# SERVER COMMANDS #
# -------------------------- #
''' Display players info '''
def et_players():
	es.dbgmsg(0, "\n")
	es.dbgmsg(0, "Players:\n")
	es.dbgmsg(0, "--------")
	es.dbgmsg(0, "Count: " + str(len(players.values())))
	for p in players.values():
		es.dbgmsg(0,\
			"\nID: " + str(p.ID) +\
			"\nName: " + p.Name +\
			"\nSteamID: " + p.SteamID)
	es.dbgmsg(0, "\n--------\n")
''' Display classes info '''
def et_classes():
	es.dbgmsg(0, "\n")
	es.dbgmsg(0, "Classes:\n")
	es.dbgmsg(0, "--------")
	es.dbgmsg(0, "Count: " + str(len(classes.values())))
	for c in classes.values():
		es.dbgmsg(0, str(c))
''' Display skills info '''
def et_skills():
	es.dbgmsg(0, "\n")
	es.dbgmsg(0, "Skills:\n")
	es.dbgmsg(0, "-------")
	es.dbgmsg(0, "Count: " + str(len(skills.values())))
	for s in skills.values():
		es.dbgmsg(0, str(s))
''' Save players stats, preferences and skills '''
def et_saveall():
	es.dbgmsg(0, "Saving players info...")
	es.delayed(2, "es_centermsg SAVING STATS, PREFERENCES AND SKILLS...")
	for p in players.values():
		if not p.IsBot():
			p.SaveStats()
			p.SavePreferences()
			p.SaveSkills()
	es.dbgmsg(0, "Players info saved")
''' Shutdown the server after saving players stats '''
def et_shutdown():
	es.centermsg("SERVER SHUTTING DOWN")
	et_saveall()
	es.dbgmsg(0, "Delay before shutting down...")
	es.delayed(3, "es_dbgmsg 0 Shutting down...;quit")